@@ -109,7 +109,7 @@ use Time::HiRes ();
 my $opt_bsd_resource = eval "use BSD::Resource; 1;";
 
 use vars qw{$VERSION};
-$VERSION = "1.46";
+$VERSION = "1.47";
 
 use warnings;
 no  warnings qw(deprecated);
@@ -632,6 +632,13 @@ sub PostEventLoop {
         $loop = 0;
         foreach my $fd (keys %PushBackSet) {
             my Danga::Socket $pob = $PushBackSet{$fd};
+
+            # a previous event_read invocation could've closed a
+            # connection that we already evaluated in "keys
+            # %PushBackSet", so skip ones that seem to have
+            # disappeared.  this is expected.
+            next unless $pob;
+
             die "ASSERT: the $pob socket has no read_push_back" unless @{$pob->{read_push_back}};
             next unless (! $pob->{closed} &&
                          $pob->{event_watch} & POLLIN);
@@ -1143,10 +1150,12 @@ sub debugmsg {
 ### Returns the string describing the peer's IP
 sub peer_ip_string {
     my Danga::Socket $self = shift;
-    return undef unless $self->{sock};
+    return _undef("peer_ip_string undef: no sock") unless $self->{sock};
     return $self->{peer_ip} if defined $self->{peer_ip};
 
-    my $pn = getpeername($self->{sock}) or return undef;
+    my $pn = getpeername($self->{sock});
+    return _undef("peer_ip_string undef: getpeername") unless $pn;
+
     my ($port, $iaddr) = Socket::sockaddr_in($pn);
     $self->{peer_port} = $port;
 
@@ -1176,6 +1185,13 @@ sub as_string {
     return $ret;
 }
 
+sub _undef {
+    return undef unless $ENV{DS_DEBUG};
+    my $msg = shift || "";
+    warn "Danga::Socket: $msg\n";
+    return undef;
+}
+
 1;
 
 # Local Variables:
